<?php
/*
* Plugin Name: Abandoned Cart Pro for WooCommerce
* Plugin URI: http://www.tychesoftwares.com/store/premium-plugins/woocommerce-abandoned-cart-pro
* Description: This plugin captures abandoned carts by logged-in users and guest users. It allows to create multiple email templates to be sent at fixed intervals. Thereby reminding customers about their abandoned orders & resulting in increased sales by completing those orders. Go to <strong>WooCommerce -> <a href="admin.php?page=woocommerce_ac_page">Abandoned Carts</a> </strong>to get started.
* Version: 8.24.0
* Author: Tyche Softwares
* Author URI: https://www.tychesoftwares.com/
* Text Domain: woocommerce-ac
* Domain Path: /i18n/languages/
* Requires PHP: 5.6
* WC requires at least: 4.0.0
* WC tested up to: 7.5.0
*
* @package Abandoned-Cart-Pro-for-WooCommerce
*/

global $ACUpdateChecker;
$ACUpdateChecker = '8.24.0';

/**
 * This is the URL our updater / license checker pings. This should be the URL of the site with EDD installed.
 * IMPORTANT: change the name of this constant to something unique to prevent conflicts with other plugins using this system.
 */ 
define( 'EDD_SL_STORE_URL_AC_WOO', 'https://www.tychesoftwares.com/' );

/**
 * The name of your product. This is the title of your product in EDD and should match the download title in EDD exactly.
 * IMPORTANT: change the name of this constant to something unique to prevent conflicts with other plugins using this system.
 */ 
define( 'EDD_SL_ITEM_NAME_AC_WOO', 'Abandoned Cart Pro for WooCommerce' );

if( ! class_exists( 'EDD_AC_WOO_Plugin_Updater' ) ) {
	// load our custom updater if it doesn't already exist
	include( dirname( __FILE__ ) . '/plugin-updates/EDD_AC_WOO_Plugin_Updater.php' );
}
/**
 * Retrieve our license key from the DB
 */ 
$license_key = trim( get_option( 'edd_sample_license_key_ac_woo' ) );
/**
 * Setup the updater
 */ 
$edd_updater = new EDD_AC_WOO_Plugin_Updater( EDD_SL_STORE_URL_AC_WOO, __FILE__, array(
		'version'   => '8.24.0',                     // current version number
		'license'   => $license_key,                // license key (used get_option above to retrieve from DB)
		'item_name' => EDD_SL_ITEM_NAME_AC_WOO,     // name of this plugin
		'author'    => 'Ashok Rane'                 // author of this plugin
		)
);

// Deactivate the Lite plugin if active. This is needed to avoid conflicts.
if ( in_array( 'woocommerce-abandoned-cart/woocommerce-ac.php', (array) get_option( 'active_plugins', array() ) ) ) {
	add_option( 'wcap_migrated_from_lite', true );
    deactivate_plugins( 'woocommerce-abandoned-cart/woocommerce-ac.php' );
}

require_once( "includes/wcap_activate_plugin.php" );
require_once( "includes/wcap_update_check.php" );
require_once( "includes/wcap_default_settings.php" );

/**
 * Woocommerce_Abandon_Cart class
 */
if ( ! class_exists( 'Woocommerce_Abandon_Cart' ) ) {

	/**
	 * It will call all the functions for the file inclusion, global variable and action & filter
	 * @since 1.0
	 */
	class Woocommerce_Abandon_Cart {
		public  $one_hour;
		public  $three_hours;
		public  $six_hours;
		public  $twelve_hours;
		public  $one_day;
		public  $one_week;
		public  $duration_range_select = array();
		public  $start_end_dates = array();

		/**
		 * It will call all the functions for the file inclusion, global variable and action & filter
		 */
		public function __construct() {

			if ( ! $this->check_is_woo_active() ) {

				add_action(
					'admin_notices',
					function() {
						$class = 'notice notice-error';
						$message = __( 'We have noticed that WooCommerce plugin is not active. Please activate it to use ' . EDD_SL_ITEM_NAME_AC_WOO. " plugin" ); //phpcs:ignore
						printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );
					}
				);
				
				if ( isset( $_GET['activate'] ) ) { //phpcs:ignore
					unset( $_GET['activate'] ); //phpcs:ignore
				}                
				add_action(
					'plugins_loaded',
					function() {
						deactivate_plugins('woocommerce-abandon-cart-pro/woocommerce-ac.php');        
					}
				);

			} else {
				add_action( 'before_woocommerce_init', function() {
					if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
						\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
						\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'orders_cache', __FILE__, true );
					}
				}, 999 );
				$this->wcap_declare_variable();
				// Load the Files.
				add_action( 'plugins_loaded', array( &$this, 'wcap_load_files' ), 5 );
				// Initialize settings.
				register_activation_hook( __FILE__, array( 'Wcap_Activate_Plugin', 'wcap_activate' ) );
				register_deactivation_hook( __FILE__, array( &$this, 'wcap_deactivate' ) );
				
				add_action( 'plugins_loaded', array( &$this, 'wcap_load_hooks' ), 6 );

				spl_autoload_register(array($this, 'loadClass'));
			}
		}

		/**
		 * Declare the common variables and the constants needed for the plugin.
		 * @since 5.0
		 */
		function wcap_declare_variable (){

			$this->one_hour     = 60 * 60;
			$this->three_hours  = 3  * $this->one_hour;
			$this->six_hours    = 6  * $this->one_hour;
			$this->twelve_hours = 12 * $this->one_hour;
			$this->one_day      = 24 * $this->one_hour;
			$this->one_week     = 7  * $this->one_day;
			$this->duration_range_select = array(
					'yesterday'         => __( 'Yesterday',    'woocommerce-ac' ),
					'today'             => __( 'Today',        'woocommerce-ac' ),
					'last_seven'        => __( 'Last 7 days',  'woocommerce-ac' ),
					'last_fifteen'      => __( 'Last 15 days', 'woocommerce-ac' ),
					'last_thirty'       => __( 'Last 30 days', 'woocommerce-ac' ),
					'last_ninety'       => __( 'Last 90 days', 'woocommerce-ac' ),
					'last_year_days'    => __( 'Last 365',     'woocommerce-ac' ) );

			$this->start_end_dates = array(
				'yesterday'     => array( 'start_date' => date( "d M Y", ( current_time('timestamp') - 24*60*60 ) ), 'end_date' => date( "d M Y", ( current_time( 'timestamp' ) - 7*24*60*60 ) ) ),

				'today'         => array( 'start_date' => date( "d M Y", ( current_time( 'timestamp' ) ) ), 'end_date' => date( "d M Y", ( current_time( 'timestamp' ) ) ) ),

				'last_seven'    => array( 'start_date' => date( "d M Y", ( current_time( 'timestamp' ) - 7*24*60*60 ) ), 'end_date' => date( "d M Y", ( current_time( 'timestamp' ) ) ) ),

				'last_fifteen'  => array( 'start_date' => date( "d M Y", ( current_time( 'timestamp' ) - 15*24*60*60 ) ), 'end_date' => date( "d M Y", ( current_time( 'timestamp' ) ) ) ),

				'last_thirty'   => array( 'start_date' => date( "d M Y", ( current_time( 'timestamp' ) - 30*24*60*60 ) ), 'end_date' => date( "d M Y", ( current_time( 'timestamp' ) ) ) ),

				'last_ninety'   => array( 'start_date' => date( "d M Y", ( current_time( 'timestamp' ) - 90*24*60*60 ) ), 'end_date' => date( "d M Y", ( current_time( 'timestamp' ) ) ) ),

				'last_year_days'=> array( 'start_date' => date( "d M Y", ( current_time( 'timestamp' ) - 365*24*60*60 ) ) , 'end_date' => date( "d M Y", ( current_time( 'timestamp' ) ) ) ) );

			/**
			 * Define The constants for plugin table names and other constants.
			 */ 
			self::wcap_define_constants_for_table_and_other();
		}

		/**
		 * Define The constants for plugin table names and other constants.
		 * @globals mixed $wpdb
		 * @since 5.0
		 */
		public static function wcap_define_constants_for_table_and_other (){

			global $wpdb;

			if ( !defined( 'WCAP_PLUGIN_VERSION' ) ) {
				define('WCAP_PLUGIN_VERSION', "8.24.0" );
			}

			if ( !defined( 'WCAP_ABANDONED_CART_HISTORY_TABLE' ) ) {
				define('WCAP_ABANDONED_CART_HISTORY_TABLE', $wpdb->prefix . "ac_abandoned_cart_history" );
			}

			if ( !defined( 'WCAP_GUEST_CART_HISTORY_TABLE' ) ) {
				define('WCAP_GUEST_CART_HISTORY_TABLE'    , $wpdb->prefix . "ac_guest_abandoned_cart_history" );
			}

			if ( !defined( 'WCAP_EMAIL_TEMPLATE_TABLE' ) ) {
				define('WCAP_EMAIL_TEMPLATE_TABLE'        , $wpdb->prefix . "ac_email_templates" );
			}

			if ( !defined( 'WCAP_NOTIFICATION_TEMPLATES_TABLE' ) ) {
				define( 'WCAP_NOTIFICATION_TEMPLATES_TABLE', $wpdb->prefix . 'ac_notification_templates' );
			}

			if ( !defined( 'WCAP_EMAIL_CLICKED_TABLE' ) ) {
				define('WCAP_EMAIL_CLICKED_TABLE'         , $wpdb->prefix . "ac_link_clicked_email" );
			}

			if ( !defined( 'WCAP_EMAIL_OPENED_TABLE' ) ) {
				define('WCAP_EMAIL_OPENED_TABLE'          , $wpdb->prefix . "ac_opened_emails" );
			}

			if ( !defined( 'WCAP_NOTIFICATION_OPENED_TABLE' ) ) {
				define( 'WCAP_NOTIFICATION_OPENED_TABLE', $wpdb->prefix . 'ac_opened_notifications' );
			}

			if ( !defined( 'WCAP_LINK_CLICKED_TABLE' ) ) {
				define( 'WCAP_LINK_CLICKED_TABLE', $wpdb->prefix . 'ac_link_clicked_notifications' );
			}

			if ( !defined( 'WCAP_EMAIL_SENT_HISTORY_TABLE' ) ) {
				define('WCAP_EMAIL_SENT_HISTORY_TABLE'    , $wpdb->prefix . "ac_sent_history" );
			}

			if ( !defined( 'WCAP_PLUGIN_FILE' ) ) {
				define('WCAP_PLUGIN_FILE'                 , __FILE__ );
			}

			if ( !defined( 'WCAP_PLUGIN_URL' ) ) {
				define('WCAP_PLUGIN_URL'                 , untrailingslashit(plugins_url('/', __FILE__)) );
			}

			if ( !defined( 'WCAP_ADMIN_URL' ) ) {
				define('WCAP_ADMIN_URL'                   , admin_url( 'admin.php' ) );
			}

			if ( !defined( 'WCAP_ADMIN_AJAX_URL' ) ) {
				define('WCAP_ADMIN_AJAX_URL'              , admin_url( 'admin-ajax.php' ) );
			}

			if ( !defined( 'WCAP_PLUGIN_PATH' ) ) {
				define('WCAP_PLUGIN_PATH'                 , untrailingslashit(plugin_dir_path(__FILE__)) );
			}
			
			if( !defined( 'WCAP_NOTIFICATIONS' ) ) {
				define( 'WCAP_NOTIFICATIONS'    , $wpdb->prefix . "ac_notifications" );
			}
			
			
			if( !defined( 'WCAP_NOTIFICATIONS_META' ) ) {
				define( 'WCAP_NOTIFICATIONS_META'    , $wpdb->prefix . "ac_notifications_meta" );
			}
			
			if( !defined( 'WCAP_TINY_URLS' ) ) {
				define( 'WCAP_TINY_URLS'    , $wpdb->prefix . "ac_tiny_urls" );
			}

			if ( !defined( 'WCAP_ATC_RULES_TABLE' ) ) {
				define('WCAP_ATC_RULES_TABLE', $wpdb->prefix . "ac_atc_rules" );
			}

			if ( !defined( 'WCAP_AC_STATS' ) ) {
				define('WCAP_AC_STATS', $wpdb->prefix . "ac_statistics" );
			}
		}

		/**
		 * Load all files needed for the plugin.
		 * @globals string $pagenow Name of the current page
		 * @since 5.0
		 */
		public static function wcap_load_files() {

			/**
			 * Load only admin side  files.
			 */
			global $pagenow;

			require_once( 'includes/connectors/class-wcap-connector.php' );
			require_once( 'includes/connectors/class-wcap-connectors-common.php' );
			require_once( 'includes/connectors/class-wcap-display-connectors.php' );
			require_once( 'includes/connectors/class-wcap-call.php' );
			require_once( 'includes/class-wcap-integrations.php' );

			require_once( "includes/admin/wcap_edd.php" );
			require_once( "includes/wcap_load_hooks.php" );
			require_once( 'includes/admin/class-wcap-print-and-csv.php' );
			require_once( "includes/wcap_load_scripts.php" );
			require_once( 'includes/admin/class-wcap-orders-listing.php' );
			
			// This condition confirm that the lite plugin active, so we need to perform further action.
			if ( in_array( 'woocommerce-abandoned-cart/woocommerce-ac.php', (array) get_option( 'active_plugins', array() ) ) || 
				( isset( $_GET ['wcap_plugin_link'] ) && 'wcap-update' == $_GET ['wcap_plugin_link'] ) ) {
				require_once( "includes/admin/wcap_import_lite_to_pro.php" );
			}

			require_once( "includes/wcap_common.php" );
			require_once( "includes/wcap_functions.php" );
			require_once( 'includes/wcap-email-functions.php' );
			require_once( "includes/wcap_ajax.php" );
			require_once( 'includes/wcap_process_base.php' );
			require_once( "includes/admin/wcap_actions_handler.php" );
			require_once( 'includes/class-wcap-webhooks.php' );
			
			// Database interaction file
			//require_once( "includes/wcap_database_layer.php" );

			// Files for SMS Settings & Reminders
			require_once( 'includes/admin/wcap_sms_reminders.php' );
			require_once( 'includes/admin/wcap_sms_settings.php' );
			
			$is_admin = is_admin();

			if ( true === $is_admin ) {
				require_once( "includes/admin/wcap_menu.php" );

				require_once( "includes/admin/class-wcap-abandoned-cart-details.php" );
				require_once( "includes/admin/wcap_add_cart_popup_modal.php" );
				// Files for Personal Data Export & Erasure
				require_once( 'includes/admin/wcap_privacy_export.php' );
				require_once( 'includes/admin/wcap_privacy_erase.php' );
				// Sent SMS List
				require_once( 'includes/admin/wcap_sent_sms_list.php' );                
			}

			if ( true === $is_admin && (
				( isset( $_GET['page'] ) && "woocommerce_ac_page" === $_GET['page'] ) || 
				( isset( $_POST[ 'option_page' ] ) && 
					( 'woocommerce_ac_settings' === $_POST[ 'option_page' ] || 
					'woocommerce_ac_license' === $_POST[ 'option_page' ] || 
					'woocommerce_sms_settings' === $_POST[ 'option_page' ] ||
					'woocommerce_ac_email_reports_frequency_settings' === $_POST[ 'option_page' ] ) 
				)
				)
				) {
				self::wcap_load_admin_side_files();

				/**
				 * Load class files.
				 */ 
				self::wcap_load_support_class_files();
			} elseif ( true === $is_admin && ( 'index.php' === $pagenow || 'admin-ajax.php' === $pagenow ) ) {
				self::wcap_load_dashboard_widget_files();
			}

			/**
			 * Load front side files.
			 */
			if( false === $is_admin ) {
				self::wcap_load_front_side_files();
			}

			require_once( "includes/classes/class_wcap_aes.php" );
			require_once( "includes/classes/class_wcap_aes_ctr.php" );

			require_once( "includes/admin/wcap_admin_recovery.php" );
			
			/**
			 * Load cart details file.
			 */ 
			$wcap_auto_cron = get_option ( 'wcap_use_auto_cron');
			if ( isset( $wcap_auto_cron ) && $wcap_auto_cron != false && '' != $wcap_auto_cron ) {
				require_once( "cron/class-wcap-send-email-using-cron.php" );
				require_once( "includes/fb-recovery/fb-recovery.php" );
			}
			// Follow-up emails file.
			require_once( 'cron/class-wcap-follow-up-emails.php' );
			/**
			 * Load FB Messenger Files
			 */
			require_once( "includes/fb-recovery/fb-recovery.php" );

			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			if ( is_plugin_active( 'woocommerce-aelia-currencyswitcher/woocommerce-aelia-currencyswitcher.php' ) ) {
				require_once( "includes/aelia-currency-switcher/wcap_aelia_currency_switcher.php" );
			}

			/**
			 * Files needed for Components folder
			 */
			require_once( 'includes/component/tracking-data/class-ts-tracker.php' );
			require_once( "includes/component/license-active-notice/ts-active-license-notice.php" );
			require_once( "includes/component/woocommerce-check/ts-woo-active.php" );

			require_once( "includes/component/tracking-data/ts-tracking.php" );
			require_once( "includes/component/deactivate-survey-popup/class-ts-deactivation.php" );

			if ( true === $is_admin ) {
				require_once( "includes/component/welcome-page/ts-welcome.php" );
			}
			require_once( "includes/component/faq-support/ts-faq-support.php" );
			require_once( "includes/wcap_all_component.php" );

			// Load file for Email Verification service.
			require_once( 'includes/class-wcap-email_verification.php' );

			do_action( 'wcap_after_load_files' );
		}

		/**
		 * Load files which needed in the admin side.
		 * @since 5.0
		 */
		public static function wcap_load_admin_side_files(){
			
			require_once( "includes/admin/wcap_add_settings.php" );
			require_once( "includes/admin/wcap_actions.php" );
			require_once( "includes/admin/wcap_display_notices.php" );
			require_once( "includes/admin/wcap_email_settings.php" );
			require_once( "includes/admin/wcap_dashboard_advanced.php" );
			require_once( "includes/admin/class-wcap-abandoned-cart-list.php" );
			require_once( "includes/admin/class-wcap-email-template-list.php" );
			require_once( "includes/admin/class-wcap-atc-templates-list.php" );
			require_once( "includes/admin/wcap_sent_email_list.php" );
			require_once( "includes/admin/wcap_email_template_fields.php" );
			require_once( "includes/admin/wcap_product_report_list.php" );
			require_once( "includes/admin/wcap_tiny_mce.php" );
			require_once( "includes/admin/wcap_localization.php" );
		}

		/**
		 * Load files needed for the front end.
		 * @since 5.0
		 */
		public static function wcap_load_front_side_files() {
			require_once( "includes/frontend/wcap_cart_updated.php" );
			require_once( "includes/frontend/wcap_order_received.php" );
			require_once( "includes/frontend/wcap_populate_cart_of_user.php" );
			require_once( "includes/frontend/wcap_on_placed_order.php" );
			require_once( "includes/frontend/wcap_coupon_code.php" );
			require_once( 'includes/frontend/class-wcap-tracking-msg.php' );
			require_once( 'includes/wcap_tiny_url.php' );
		}

		/**
		 * Load the supporting class files.
		 * @since 5.0
		 */
		public static function wcap_load_support_class_files(){

			require_once( "includes/classes/class_wcap_manual_email.php" );
			require_once( "includes/classes/class-wcap-send-manual-email.php" );
			require_once( "includes/classes/class_wcap_dashboard_adv_report_action.php" );
			require_once( "includes/classes/class_wcap_abandoned_orders_table.php" );
			require_once( "includes/classes/class_wcap_abandoned_trash_orders_table.php" );
			require_once( "includes/classes/class_wcap_templates_table.php" );
			require_once( "includes/classes/class_wcap_sent_emails_table.php" );
			
			require_once( "includes/classes/class_wcap_product_report_table.php" );
			require_once( "includes/classes/class_wcap_atc_dashboard.php" );
			require_once( 'includes/classes/class_wcap_sms_templates_table.php' );
			require_once( 'includes/classes/class_wcap_sent_sms_table.php' );
			require_once( 'includes/classes/class-wcap-atc-templates-table.php' );
		}

		/**
		 * Loads the dashboard widget files.
		 * @since 5.0
		 */
		public static function wcap_load_dashboard_widget_files() {
			require_once( "includes/admin/wcap_dashboard_widget.php" );
			require_once( "includes/classes/class_wcap_dashboard_widget_report.php" );
			require_once( "includes/classes/class_wcap_dashboard_widget_heartbeat.php" );
		}

		/**
		 * It will load all the hooks needed for the plugin.
		 * @since 5.0
		 */
		static function wcap_load_hooks (){
			Wcap_Load_Hooks::wcap_load_hooks_and_filters();
			do_action( 'wcap_after_load_hooks' );
		}

		/**
		 * Actions to be run when deactivating the plugin.
		 *
		 * @since 9.0
		 */
		function wcap_deactivate() {
			// Send reminders.
			if ( false !== as_next_scheduled_action( 'woocommerce_ac_send_email_action' ) ) {
				as_unschedule_action( 'woocommerce_ac_send_email_action' );
			}
		}

		/**
		 * Autoloader for classes
		 *
		 * @since 9.0
		 */

		function loadClass( $className ){
			$ds = DIRECTORY_SEPARATOR;
			$plugin_dir = realpath( plugin_dir_path( __FILE__ ) ).$ds;            
			$directories = array(
				'includes'.$ds,
				'includes'. $ds .'admin'.$ds,
				'includes'. $ds .'classes'.$ds,
				'includes'. $ds .'component'.$ds,
				'includes'. $ds .'frontend'.$ds,
				'includes'. $ds .'libraries'.$ds,
				'includes'. $ds .'models'.$ds
			);
			$classPath = trim( $className, 'AC\\' );            
			$folderArray = explode( '\\', $classPath );
			$fileName = strtolower( str_replace( '_','-',array_pop( $folderArray ) ) ).'.php';
			$fileFolder = strtolower( implode( $ds , $folderArray) ).$ds;            

			foreach ( $directories as $directory ){

				$fullPath = $plugin_dir.$directory.$fileFolder.$fileName;
				
				if (file_exists($fullPath)) {
					require_once($fullPath);
					//return;
				}
			}            
		}
		/**
		 * Check if woocommerce is active for the site or network
		 */
		public function check_is_woo_active() {
			$is_active = false;
			if ( ! function_exists('is_plugin_active') ) {
				include_once(ABSPATH . 'wp-admin/includes/plugin.php');
			}
			$network_check = true;
			$network_check = apply_filters( 'wcap_do_network_check', $network_check );
			if ( $network_check && is_multisite() ) {
				$is_active =  is_plugin_active_for_network('woocommerce/woocommerce.php');
			} else {
				$is_active =  is_plugin_active('woocommerce/woocommerce.php');
			}
			return $is_active;
		}
	}
}
$woocommerce_abandon_cart = new Woocommerce_Abandon_Cart();


